#include <stdlib.h> // for exit()
#include <math.h>   // for fabs(), sqrt()  pow()
#include <strstream>
#include <iostream>
#include <string>
#include <iomanip>
#include <new>
#include "d_matrix.h"

using namespace std;

int Matrix::Width = 0;
int Matrix::Precision = 4;

Matrix::Matrix(size_t y, size_t x)
 { if (x && y)
   { try { dim = new double[(m = y) * (n = x)]; }
     catch (bad_alloc) { throw bad_matrix(7); }
     memset(dim, 0, sizeof(double) * m * n); 
   }
   else { m = n = 0; dim = (double*)0; }
 }

Matrix::Matrix(const Matrix &A)
 { if (A.dim)
   { try { dim = new double[(m = A.m) * (n = A.n)]; }
     catch (bad_alloc) { throw bad_matrix(7); }
     memmove(dim, A.dim, sizeof(double) * m * n);
   }
   else { m = n = 0; dim = (double*)0; }
 }

Matrix::Matrix(char  * d)
 { size_t y, x;
   istrstream mem(d); mem>>y; mem>>x;
   if (x && y)
   { try { dim = new double[(m = y) * (n = x)]; }
     catch (bad_alloc) { throw bad_matrix(7); }
     memset(dim, 0, sizeof(double) * m * n);
   }
   else { m = n = 0; dim = (double*)0; return; }

   for(size_t i = 0,j; i<y; i++)
      for(j = 0; j<x; j++) 
         if( !(mem>>(*this)(i, j)) ) return; 
 }

Matrix::Matrix(istream &inp)
 { size_t y, x; inp>>y; inp>>x;
   if (x && y)
   { try { dim = new double[(m = y) * (n = x)]; }
     catch (bad_alloc) { throw bad_matrix(7); }
     memset(dim, 0, sizeof(double) * m * n);
   }
   else { m = n = 0; dim = (double*)0; return; }
   
   for(size_t i = 0,j; i<y; i++)
      for(j = 0; j<x; j++)
         if( !(inp>>(*this)(i, j)) ) return;
 }

 const char* Matrix::bad_matrix::error() const
 { static const char *err_mas[]  =  {
   /*  0  */  "The matrixes must have identical dimansions",  
   /*  1  */  "Overrun. Effort of access to non-existent element",    
   /*  2  */  "There was breached the rule of dimension of the matrixes during their multiplication",  
   /*  3  */  "The degree must be larger than zero or -1",  
   /*  4  */  "The matrix must be squarte", 
   /*  5  */  "The insertion is impossible because the covering of the basic matrix took place", 
   /*  7  */  "Memory allotment is impossible", 
   /*  8  */  "The matrix is singular", 
   /*  9  */  "The zero element in Gauss' transformation",  
   /*  10 */  "The effort of giving the matrix itself" };  
   if (err_num>=0 && err_num<=10) return err_mas[err_num]; else return 0;
 }

Matrix& Matrix::operator = (const Matrix &A)
 { if (this == &A) throw bad_matrix(10);
   if (A.dim)
   { if(!dim)
     { try { dim = new double[(m = A.m) * (n = A.n)]; }
       catch (bad_alloc) { throw bad_matrix(7); }
     }
     if(m != A.m || n != A.n) throw bad_matrix(0);
     memmove(dim, A.dim, sizeof(double) * m * n);
   }
   return *this;
 }

Matrix& Matrix::operator = (const double &f)
 { if (f && n != m) throw bad_matrix(4);
   for(size_t i = 0,j; i<m; i++)
      for(j = 0; j<n; j++)
         if(i == j) (*this)(i, j) = f;
         else (*this)(i, j) = 0;
   return *this;                          
 }

Matrix Matrix::operator^(int p) const // exponentiation
 { if (p>0)
   { Matrix B = *this;
     for( ;p>1; p--) B = B * (*this);
     return B;
   }
   if (p != -1) throw bad_matrix(3);
   // calculation of the inverse matrix using method of Gauss' transformations
   if (n != m) throw bad_matrix(4);
   Matrix B(m, 2 * n);
   B.insert(*this);
   for(size_t i = 0; i<B.m; i++) B(i, i + B.m) = 1;
   for(size_t i = 0; i<B.m; i++) B.Gauss1(i, i);
   return B.extract(B.m, B.m, 0, B.m);
 }

Matrix Matrix::operator!()  // transposition
 { Matrix A(n, m);
   for(size_t i = 0,j; i<m; i++) 
      for(j = 0; j<n; j++) A(j, i) = (*this)(i, j);
   return A;
 }

double Matrix::tr() const   // the matrix's trace
 { if (n != m) throw bad_matrix(4);
   double f = 0; for(size_t i = 0; i<m; i++) f += (*this)(i, i);
   return f;
 }

double Matrix::det() const  // finds recursively the matrix's determinant
 { if (n != m) throw bad_matrix(4);
   if (n == 1) return (*this)(0, 0);
   for(size_t i = 0; i<m; i++) if ((*this)(i, 0))
   {  static Matrix M;
      M<<= *this;
      double d = M(i, 0) * (i%2 ? -1 : 1);
      return d * M.Gauss1(i, 0).minor(i, 0).det();
   }
   return 0;
 }

bool Matrix::positive() const
 { if (n != m) throw bad_matrix(4);
   for(size_t i = 1; i <= m; i++)
      if (extract(i, i).det()<= 0) return 0;
   return 1;
 }

bool Matrix::negative() const
 { if (n != m) throw bad_matrix(4); 
   for(size_t i = 1; i<= m; i++)
      if(( i%2 ? -1 : 1) * extract(i, i).det()<= 0) return 0;
   return 1;
 }

double Matrix::norma1() const
 { double t = 0;
   for(size_t i = 0,j; i<m; i++)
   {   double s = 0;
       for(j = 0; j<n; j++) s += fabs((*this)(i, j));
       if (s>t) t = s;
   }
   return t;
 } 

double Matrix::norma2() const
 { double s = 0;
   for(size_t i = 0,j; i<m; i++)
      for(j = 0; j<n; j++) s += pow((*this)(i, j), 2); 
   return sqrt(s);
 } 

double Matrix::cond(int norm) const
 { if (n != m) throw bad_matrix(4); 
   if (norm == 1) return this->norma1() * ((*this)^-1).norma1();
   else return this->norma2() * ((*this)^-1).norma2();
 }

void Matrix::maxN(size_t &mi, size_t &mj, size_t ly, size_t lx, size_t y, size_t x) const
 { if (!ly) ly = m;
   if (!lx) lx = n; 
   const Matrix &A = *this;
   double m = A(mi = y, mj = x);
   for(size_t i = 0,j; i<ly; i++)
      for(j = 0; j<lx; j++)
         if (A(i + y, j + x) > m)
         { m = A(i + y, j + x);
           mi = i + y;
           mj = j + x;
         }
 }

Matrix& Matrix::Gauss1(size_t y, size_t x)
 { Matrix &A = *this;
   if (!A(y, x)) throw bad_matrix(9);
   for(size_t i = 0,j; i<m; i++)
      for(j = 0; j<n; j++)
         if (i != y && j != x) A(i, j) -= A(y, j) * A(i, x)/A(y, x);
   for(size_t j = 0; j<n; j++)
      if (j != x) A(y, j) /= A(y, x);
   for(size_t i = 0; i<m; i++) A(i, x) = 0;
   A(y, x) = 1;
   return *this;
 }

Matrix& Matrix::Gauss2(size_t y, size_t x)
 { Matrix &A = *this;
   if (!A(y, x)) throw bad_matrix(9);
   for(size_t i = y + 1, j; i<m; i++)
      for(j = x + 1; j<n; j++)
         A(i, j) -= A(y, j) * A(i, x) / A(y, x);
   for(size_t i = y + 1; i<m; i++) A(i, x) = 0;
   return *this;
 }

Matrix Matrix::minor(size_t y, size_t x) const  // returns the matrix without raw y and column x 
 { Matrix A(m-1, n-1);
   for(size_t i = 0, in = 0, j, jn; i<m; i++)
      if (i != y)
      { for(j = 0, jn = 0; j<n; j++)
           if (j != x) A(in, jn++) = (*this)(i, j);
           in++;
      }
   return A;
 }

double& Matrix::operator() (size_t y, size_t x) const
 { if (x >= n || y >= m) throw bad_matrix(1);
   return  * (dim + n * y + x);
 }

Matrix& Matrix::insert(const Matrix &A, size_t y, size_t x)  // insertion
 { if (y + A.m>m || x + A.n>n) throw bad_matrix(5);
   for(size_t i = 0,j; i<A.m; i++)
      for(j = 0; j<A.n; j++) (*this)(i + y, j + x) = A(i, j);
   return *this;
 }

Matrix Matrix::extract(size_t ly, size_t lx, size_t y, size_t x) const // extraction
 { if (y + ly > m || x + lx > n) throw bad_matrix(6);
   Matrix A(ly, lx);
   for(size_t i = 0,j; i<ly; i++) 
      for(j = 0; j<lx; j++) A(i, j) = (*this)(i + y, j + x);
   return A;
 }

Matrix& Matrix::newsize(size_t y, size_t x)
 { delete []dim;
   if (x && y)
   { try { dim = new double[(m = y) * (n = x)]; }
   catch (bad_alloc) { throw bad_matrix(7); }
     memset(dim, 0, sizeof(double) * m * n);
   }
   else { m = n = 0; dim = (double*)0; }
   return *this;
 }

Matrix operator+(const Matrix &A, const Matrix &B)
{ if (A.n != B.n || A.m != B.m) throw Matrix::bad_matrix(0);
   Matrix C = A;
   for(size_t i = 0,j; i<A.m; i++)
      for(j = 0; j<A.n; j++) C(i, j) += B(i, j);
   return C;
 }

Matrix operator-(const Matrix &A, const Matrix &B)
 { if (A.n != B.n || A.m != B.m) throw Matrix::bad_matrix(0);
   Matrix C = A;
   for(size_t i = 0,j; i<A.m; i++)
      for(j = 0; j<A.n; j++) C(i, j) -= B(i, j);
   return C;
 }

Matrix operator*(const Matrix &A, const Matrix &B)
 { if (A.n != B.m) throw Matrix::bad_matrix(2);
   Matrix C(A.m, B.n);
   for(size_t i = 0,j, k;i<C.m;i++)
      for(j = 0;j<C.n;j++) 
         for(k = 0; k<A.n; k++) C(i, j) += A(i, k) * B(k, j);
   return C;
 }

Matrix operator*(const double &f, const Matrix &A) // multiplication to the number
 { Matrix B = A; 
   for(size_t i = 0,j; i<A.m; i++) 
      for(j = 0; j<A.n; j++) B(i, j) *= f;
   return B;
 }

bool operator<=(const Matrix &A, const Matrix &B)
 { if (A.n != B.n || A.m != B.m) throw Matrix::bad_matrix(0);
   bool fl = 1;
   for(size_t i = 0, j; i<A.m; i++) 
      for(j = 0; j<A.n; j++) 
         if (A(i, j)>B(i, j)) fl = 0;
   return fl;
 }

ostream& operator<<(ostream &out, const Matrix &A)
 { for(size_t i = 0,j; i<A.size_i(); i++)
   {   for(j = 0; j<A.size_j(); j++)
   out<<setw(Matrix::Width)<<setprecision(Matrix::Precision)
          <<setiosflags(ios::fixed)<<A(i, j); out<<endl;
   }
   return out;
 }

bool operator>>(istream &inp, Matrix &A)
 { for(size_t i = 0,j; i<A.size_i(); i++)
      for(j = 0; j<A.size_j(); j++)
         if( !(inp>>A(i, j)) ) return 1;
   return 0;
 }

Matrix& Matrix::operator<<=(double d)
 { for(size_t i = 0,j; i<size_i(); i++)
      for(j = 0; j<size_j(); j++) (*this)(i,j)=d;
   return *this;
 }

void Matrix::swap_line(size_t l1, size_t l2)
 { if (l1 == l2) return; 
   double b;
   for(size_t j = 0; j<n; j++)
   {    b = (*this)(l1, j);
        (*this)(l1, j) = (*this)(l2, j);
        (*this)(l2, j) = b;
   }
 }

void Matrix::swap_column(size_t c1, size_t c2)
 { if (c1 == c2) return;
   double b;
   for(size_t i = 0; i<m; i++)
   {    b = (*this)(i, c1);
        (*this)(i, c1) = (*this)(i, c2);
        (*this)(i, c2) = b;
   }
 }

bool Matrix::normir() // normalization of the matrix
{ Matrix &A = *this;
  double s, max = 0;
  size_t i = 0,j;
  for(; i<m; i++)
  {    for(j = 0, s = 0; j<n; j++) s += A(i, j);
       if (s>max) max = s;
  }
  for(j = 0; j<n; j++)
  {    for(i = 0, s = 0; i<m; i++) s += A(i, j);
       if (s>max) max = s;
  }
  if (max == 0) return 1;
  max += max/100;
  A = (1/max) * A;
  return 0;
}

Matrix Matrix::to3d() const
{ // method of elementary transformations with selection of the main element 
  if (n != m) throw bad_matrix(4); 
  Matrix A = *this,  N(n, n),  P(n, n);
  for(size_t q = 0, i, j; q<n-2; q++) // the first iteration
  {    P = 1;
       A.maxN(i, j, n-q-1, 1, q+1, q);
       P.swap_line(q + 1, i);
       if (A(q + 1, q))
       {   N = 1;
           A = P * A;
           for(i = q + 2; i<n; i++) N(i, q+1) = -A(i, q) / A(q+1, q);
           A = N * A;
           for(i = q + 2; i<n; i++) N(i, q+1) = -N(i, q+1);
           A = A * P * N;
       }
  }
  for(size_t q = 0; q<n-2; q++) // the second iteration
     if(A(q, q + 1))
     {    N = 1; 
          for(size_t j = q + 2; j<n; j++) N(q+1, j) = A(q, j) / A(q, q+1);
          A = N * A; 
          for(size_t j = q + 2; j<n; j++) N(q+1, j) = -N(q+1, j);
          A = A * N;
     }
  return A;
}

Vector Matrix::GaussMethod() const
{ Matrix A = *this;
  size_t  * ni = new size_t[m], im, jm, i = 0, j;
  double s;
  Vector x(m), xr(m), y(1, m + 1);
  for( ; i<m; i++) ni[i] = i;
  for(i = 0; i<m-1; i++) // the forward trace
  {    A.maxN(im, jm, m-i, m-i, i, i);
       A.swap_line(i, im);
       if (i != jm)
       {   A.swap_column(i, jm);
           j = ni[jm];
           ni[jm] = ni[i];
           ni[i] = j;
       }
       A.Gauss2(i, i);
  }
  for(long ii = long(m-1); ii >= 0; ii--) // the reverse trace
  {    if (!A(ii, ii)) throw bad_matrix(8);
       s = 0;
       for(size_t j = ii + 1; j<m; j++) s += A(ii, j) * x(j);
       x(ii) = (A(ii, n-1)-s)/A(ii, ii);
  }
  for(i = 0; i<m; i++) xr(ni[i]) = x(i);
  delete []ni;
  return xr;
}

Vector Matrix::ZeydlMethod(double eps, int &ni) const
{ Matrix A = *this; // transformation to the form of simple iteration
  size_t i = 0,j;
  int k = 0;
  for(; i<m; A(i, m) /= A(i, i), A(i, i) = 0, i++)
     for(j = 0; j<m; j++)
        if (i != j) A(i, j) = -A(i, j)/A(i, i);
  Vector xn(m), x = A.column(m);

  for(double nr, nrn ; k<ni ; x = xn, nr = nrn, k++)
  {   for(i = 0; i<m; i++) // iteration
      {    xn(i) = A(i, m);
           for(j = 0; j<i; j++) xn(i) += A(i, j) * xn(j);
           for(j = i; j<m; j++) xn(i) += A(i, j) * x(j);
      }
      nrn = (xn-x).norma1();
      nr = (!k) ? nrn : nr;
      if (k) if (nrn > nr) { ni = 0; return xn; } // the method is dispersing
      else if ( nrn < eps) break; // the approximation is found
  }
  ni = k;
  return xn;
}

Vector Matrix::ReflectionMethod() const
{ Matrix A = this->extract(m, m), I(m, m), H(m, m);
  I = 1;
  Vector b = this->column(n-1), om(m), f(m);
  double s;
  size_t i = 0, j;
  for( ; i<m-1; i++) // the forward trace
  {    f = A.column(i);
       for(j = 0; j<i; j++) f(j) = 0;
       om = f-sqrt(((!f) * f)(0, 0)) * I.column(i);
       om = (1/om.norma2()) * om;
       H = I - 2 * om * (!om);
       A = H * A;
       b = H * b;
  }
  for(long ii = long(m-1); ii >= 0; ii--) // the reverse trace
  {   if (!A(ii, ii)) throw bad_matrix(8);
      for(j = ii + 1, s = 0; j<m; j++) s += A(ii, j) * f(j);
      f(ii) = (b(ii)-s) / A(ii, ii);
  }
  return f;
}

ostream& operator<<(ostream& out, const Matrix::bad_matrix &err)
{ return out<<"Matrix error: "<<err.error()<<" !"<<endl; }
