#pragma once

#include <fstream>

using namespace std;

//-----------------------------------------------------[ Matrix ]----------
class Matrix;
typedef Matrix Vector;

class Matrix {
 
private:
   double *dim;    // pointer to the array of the matrix
   size_t m, n;    // dimension
   static int Width,        // width in output
              Precision;    // precision in output
public:

// constructors and destructors
   Matrix() { dim=(double*)0; m=n=0; }
   Matrix(size_t, size_t=1);
   Matrix(char*);
   Matrix(istream&);
   Matrix(const Matrix&);
   ~Matrix() { delete []dim; }
   static void field(int w, int p=2) { Width=w; Precision=p; }

// exception class	
   class bad_matrix
   { int err_num;
	 public:
		 bad_matrix(int i) :err_num(i) {}
	     const char* error() const;
   };
   
// acsess to the elements
   size_t size_i() const { return m; }  // quantity of rows
   size_t size_j() const { return n; }  // quantity of columns

// overloaded operations and functions
   Matrix& operator=(const Matrix&);
   Matrix& operator=(const double&);
   Matrix& operator<<=(const Matrix &A) { return newsize(A.m, A.n) = A; } // absolute equation of matrixes
   Matrix& operator<<=(double);  // filling the whole matrix with one number 

   // returns global position of the largest number in matrix's block
   void maxN(size_t&, size_t&, size_t=0, size_t=0, size_t=0, size_t=0) const;

   // returns vector-raw of the matrix 
   Vector line(size_t i) const { return extract(1,n,i,0); }

   // returns vector-column of the matrix 
   Vector column(size_t j) const { return extract(m,1,0,j); }

   double& operator() (size_t, size_t) const;                   // access to
   double& operator() (size_t i) const { return (*this)(i,0); } // element

   Matrix  operator^(int) const; // exponentiation
   Matrix  operator!();          // transposition
   bool positive() const;        // is it positive or negative 
   bool negative() const;   
   double tr() const;            // matrix's trace 
   double det() const;           // matrix's determinant

   double norma1() const; // norm of the matrix: maximum of the sums of the raws
   double norma2() const; // norm of the matrix: the root of the sums elements' squares 
   bool normir();         // such a normalization of the matrix , that ||A||<1
   double cond(int) const;  // search of a measure of conditionality of the matrix
   Matrix& Gauss1(size_t, size_t);      // execution of Gauss's transformation (in order to get inverse and unitary matrix)
   Matrix& Gauss2(size_t, size_t);      // execution of Gauss's transformation (in order to get hightriangular matrix)
   Matrix  minor(size_t, size_t) const; // returns indicating minor of the matrix

   Matrix& insert(const Matrix&, size_t=0, size_t=0);         // to insert the block of the matrix
   Matrix  extract(size_t, size_t, size_t=0, size_t=0) const; // to extract the block of the matrix
   Matrix& newsize(size_t, size_t=1); // to set new dimensions
   void swap_line(size_t, size_t);    // matrix's raws exchange  
   void swap_column(size_t, size_t);  // matrix's columns exchange

   friend Matrix operator+(const Matrix&, const Matrix&);
   friend Matrix operator-(const Matrix&, const Matrix&);
   friend Matrix operator*(const Matrix&, const Matrix&);
   friend Matrix operator*(const double&, const Matrix&);
   friend bool   operator<=(const Matrix&, const Matrix&);
   friend ostream& operator<<(ostream&, const Matrix&);
                                 // elemental streaming output of the matrix
   friend bool operator>>(istream&, Matrix&);
                                 // streaming input of the existing matrix 
                                 // 0 - without errors, 1 - there was an error
   Matrix to3d() const;
          // transformation to the triangular form
          // using the method of elementary transformations with selection of the main element
   Vector GaussMethod() const;
          // solution of the system of linear equations using the method of serial exclusion
          // of unknowns with selection of the main element in the whole matrix
   Vector ZeydlMethod(double, int&) const;
          // solution of the system of linear equations using the method of Zeidel
          // If the method disperses then the last parameter returns zero
   Vector ReflectionMethod() const;
          // solution of the system of linear equations using the method of reflections  

}; /* the end of Matrix */ 

ostream& operator<<(ostream&, const Matrix::bad_matrix&);

//----------------------------------------------------------------------------


