// -----------------------------------------+
//     SimplexIteration |
// -----------------------------------------+

#include <stdlib.h> //  exit()  atof()
#include <math.h>   //  fabs()
#include <string>
#include <iomanip>
#include <strstream>
#include "simplex.h"

using namespace std;

double SimplexIteration::mash_eps=1e-10;

const char *SimplexIteration::bad_simplex::err_mas[] = {
   /* 0  */ " ",
   /* 1  */ "H     ",
   /* 2  */ "     ",
   /* 3  */ "   ",
   /* 4  */ "    ",
   /* 5  */ "   ",
   /* 6  */ "     ",
   /* 7  */ "H      ",
   /* 8  */ "     ",
   /* 9  */ "p p     ",
   /* 10 */ "p p  ",
   /* 11 */ " p     ",
   /* 12 */ " p    p",
   /* 13 */ " p   ",
   /* 14 */ "     ",
   /* 15 */ "       ",
   /* 16 */ "\"\"       ",
   /* 17 */ "    ",
   /* 18 */ "      ",
   /* 19 */ "     "  };

//    
SimplexIteration::SimplexIteration()
 {
   A=0; d=0; B=0; w=0; var=0; name=0; out_f=0;
 } //     

//    
SimplexIteration::SimplexIteration(const char *fn)
 { ifstream inp(fn); SimplexIteration::bad_simplex e; 
   if(!inp) { e.add_err(1); throw e; }
   inp.width(30); char *buf=new char[30];
   crit=Vacant; name=out_f=0; var=0; B=0;
   iter=0; kln=klm=base=0;
   Width=10; Precision=4; size_t i,j,k; Start_Iter=0; End_Iter=0;
   OnlyDataTable=OnlyOptimalTable=NoTable=0; Stop=0;

   while(!inp.eof())
   { inp>>buf;
     if (!stricmp(buf,"Size:") || !stricmp(buf,":"))
	{ size_t m=0,n=0; inp>>m>>n;
          if (!m || !n || !inp) { e.add_err(4); throw e; }
          if (m>MAX_LINES || n>MAX_COL) { e.add_err(4); throw e; }
          if (m>n) { e.add_err(11); continue; }
	  m++; n++; A.newsize(m,n); d.newsize(2,n);
          B=new size_t[m]; w=new size_t[n+2]; w+=2;
          continue; }
     if (!stricmp(buf,"Base:") || !stricmp(buf,":"))
	{ if (!B) { e.add_err(16); continue; }
          size_t b; inp>>b;
          if (!b || b>=A.size_j()) { e.add_err(17); continue; }
          base=b; continue; }
     if (!stricmp(buf,"OnlyDataTable") || !stricmp(buf,"__"))
	{ OnlyDataTable=1; continue; }
     if (!stricmp(buf,"OnlyOptimalTable") || !stricmp(buf,"__"))
	{ OnlyOptimalTable=1; continue; }
     if (!stricmp(buf,"NoTable") || !stricmp(buf,"_"))
	{ NoTable=1; continue; }
     if (!stricmp(buf,"Accuracy:") || !stricmp(buf,":"))
        { inp>>Width>>Precision;
          if (Precision>=Width || Precision<0) { e.add_err(18); continue; }
	  continue; }
     if (!stricmp(buf,"Print:") || !stricmp(buf,":"))
        { inp>>Start_Iter>>End_Iter;
          if (End_Iter<Start_Iter || End_Iter<0) { e.add_err(19); continue; }
	  continue; }
     if (!stricmp(buf,"Stop:") || !stricmp(buf,":"))
        { inp>>Stop; continue; }
     if (!stricmp(buf,"Var_names:") || !stricmp(buf,":"))
	{ var=new char*[A.size_j()]; var[0]=0;
          for(j=1;j<A.size_j();j++)
	     { inp>>buf; var[j]=new char[strlen(buf)+1]; strcpy(var[j],buf); }
	  continue; }
     if (!stricmp(buf,"Matrix:") || !stricmp(buf,":"))
	{ if (!B) { e.add_err(6); break; }
	  for(j=1;j<A.size_j();j++) // .  
             { if (!(inp>>buf)) { e.add_err(5); break; }
               if (*buf=='M') A(0,j)=M;
               else if (*buf=='-' && buf[1]=='M') A(0,j)=-M;
               else { A(0,j)=atof(buf);
                      if (!A(0,j) && *buf!='0') { e.add_err(5); break; }
                    }
             }
          for(i=1; i<A.size_i(); i++)
	     for(j=0; j<A.size_j(); j++)
                { if ( !(inp>>A(i,j)) ) { e.add_err(5); break; }
                  if (!j && A(i,j)<0) { e.add_err(9); break; }
                }
	  continue; }
     if (!stricmp(buf,"Aim:") || !stricmp(buf,":"))
	{ inp>>buf; if(!stricmp(buf,"max") ||
		       !stricmp(buf,"")) crit=MAX;
		    else if(!stricmp(buf,"min") ||
			    !stricmp(buf,"")) crit=MIN;
			 else e.add_err(2);
	  continue; }
     if (!stricmp(buf,"Name:") || !stricmp(buf,":"))
	{ if (!name)
	     { inp.unsetf(ios::skipws); char *nb=new char[80],*c=nb; i=0;
	       do { inp>>*c; i++; } while(*c++!='\n' && i<80); c--; *c=0;
	       inp.setf(ios::skipws); name=new char[strlen(nb)];
	       strcpy(name,nb+1); delete []nb; }
	  else e.add_err(3);
	  continue; }
     if (!stricmp(buf,"File:") || !stricmp(buf,":"))
	{ inp>>buf; out_f=new char[strlen(buf)+1]; strcpy(out_f,buf); continue; }
     if (!stricmp(buf,"Format:") || !stricmp(buf,":"))
        { /*    */ }
   }
   if (crit==Vacant) e.add_err(14); if (!out_f) e.add_err(15);
   //   
   ReWidth();
   //   
   for(i=1; i<A.size_i(); i++)
      { for(j=1,B[i]=0; j<A.size_j(); j++)
	   if (A(i,j)==1)
	      { for(B[i]=j,k=1; k<A.size_i(); k++)
		   if (k!=i) if (A(k,j)) { B[i]=0; break; }
		if (B[i]) break;
	      }
	if (!B[i]) e.add_err(8);
      }
   B[0]=0;
   delete []buf;
   inp.close();
   if (e) throw e;
 } //     

SimplexIteration::~SimplexIteration()
 { if (B) delete B;
   if (name) delete []name;
   if (out_f) delete []out_f;
   if (w) delete [] (w-2);
   if (var)
   { for(size_t j=1; j<A.size_j(); j++) if (var[j]) delete [] var[j];
     delete [] var;
   }
 }

//   
SimplexIteration::SimplexIteration(const SimplexIteration &I)
{ crit=I.crit; A=I.A; d=I.d; kln=I.kln; klm=I.klm; base=I.base; iter=I.iter;
  Width=I.Width; Precision=I.Precision;
  Start_Iter=I.Start_Iter; End_Iter=I.End_Iter; Stop=I.Stop;
  OnlyDataTable=I.OnlyDataTable; OnlyOptimalTable=I.OnlyOptimalTable;
  NoTable=I.NoTable;

  out_f=new char[strlen(I.out_f)+1]; strcpy(out_f,I.out_f);
  B=new size_t[A.size_i()]; memmove(B,I.B,A.size_i()*sizeof(size_t));
  w=new size_t[A.size_j()+2]; memmove(w,I.w-2,(A.size_j()+2)*sizeof(size_t)); w+=2;
  if (I.name) { name=new char[strlen(I.name)+1]; strcpy(name,I.name); }
  else name=0;
  if (I.var) { var=new char*[A.size_j()]; var[0]=0;
               for(size_t j=1; j<A.size_j(); j++)
               { var[j]=new char[strlen(I.var[j])+1];
                 strcpy(var[j],I.var[j]);
               }
             } else var=0;
}

//   
SimplexIteration& SimplexIteration::operator=(const SimplexIteration &I)
{ crit=I.crit; Start_Iter=I.Start_Iter; End_Iter=I.End_Iter; Stop=I.Stop;
  OnlyDataTable=I.OnlyDataTable; OnlyOptimalTable=I.OnlyOptimalTable;
  NoTable=I.NoTable;
  Width=I.Width; Precision=I.Precision;
  A<<=I.A; d<<=I.d; kln=I.kln; klm=I.klm; base=I.base; iter=I.iter;

  if (out_f) delete []out_f;
  out_f=new char[strlen(I.out_f)+1];
  strcpy(out_f,I.out_f);

  if (B) delete B;
  B=new size_t[A.size_i()]; memmove(B,I.B,A.size_i()*sizeof(size_t));

  if (w) delete [] (w-2); w=new size_t[A.size_j()+2];
  memmove(w,I.w-2,(A.size_j()+2)*sizeof(int)); w+=2;

  if (I.name) { if (name) delete []name;
                name=new char[strlen(I.name)+1];
                strcpy(name,I.name); } else name=0;
  if (I.var)
     { size_t j=1;
       if (var) { for(; j<A.size_j(); j++) if (var[j]) delete [] var[j];
                  delete [] var; }
       var=new char*[A.size_j()]; var[0]=0;
       for(j=1; j<A.size_j(); j++)
          { var[j]=new char[strlen(I.var[j])+1]; strcpy(var[j],I.var[j]); }
     } else var=0;

  return *this;
}

     //   
void SimplexIteration::ReWidth()
{  w[-2]=w[0]=Width; w[-1]=5; for(size_t j=1;j<A.size_j();j++) w[j]=Width;
   if (var) for(size_t j=1; j<A.size_j(); j++)
               if(strlen(var[j])>w[j]) w[j]=strlen(var[j]);
}

     //    
void SimplexIteration::calculate_d()
{ for(size_t j=0,i; j<A.size_j(); j++)
     { d(0,j)=d(1,j)=0;
       for(i=1; i<A.size_i(); i++)
          if (fabs(A(0,B[i]))==M) d(0,j)+=A(i,j)*((crit==MAX)?-1:1);
          else d(1,j)+=A(i,j)*A(0,B[i]);
       if (!j) continue;
       if (fabs(A(0,j))==M) d(0,j)-=(crit==MAX)?-1:1;
       else d(1,j)-=A(0,j);
       if (fabs(d(1,j))<epsilon()) d(1,j)=0;
     }
}
      //      
      //  0   
size_t SimplexIteration::be_not_optimal()
{ size_t f=0;
  for (size_t j=1; j<A.size_j(); j++)
  if ( (crit==MAX && (d(0,j)<0 || !d(0,j) && d(1,j)<0)) ||
       (crit==MIN && (d(0,j)>0 || !d(0,j) && d(1,j)>0)) )
       if (!f) f=j;
       else if ( fabs(d(0,j))>fabs(d(0,f)) ||
                 fabs(d(0,j))==fabs(d(0,f)) && fabs(d(1,j))>fabs(d(1,f)) ) f=j;

  if (!f && base) //   
     for (size_t j=1; j<=base; j++)
         { if (d(0,j)==0 && d(1,j)==0)
              { //       ?
                f=1; for(size_t i=1; i<A.size_i(); i++) if (B[i]==j) f=0;
                //      ?
                if (f) for(size_t i=1; i<A.size_i(); i++)
                       if (B[i]>base && A(i,j)>0) { f=j; B[0]=1; break; }
              }
           if (B[0]==1) break;
         }
  kln=f; return f;
}

     //      
     //  0    (.  )
size_t SimplexIteration::be_solved()
{ size_t f=0; double min=0,tst;
  for(size_t i=1; i<A.size_i(); i++)
  if (A(i,kln)>0 && (!base || !B[0] || B[i]>base))
     { tst=A(i,0)/A(i,kln); if (!f || tst<min) { f=i; min=tst; } }
  klm=f; return f;
}

     //   
void SimplexIteration::operator++(int)
{ B[klm]=kln; iter++;
  size_t i=1,j;
  for(;i<A.size_i();i++)
     if (i!=klm) for(j=0;j<A.size_j();j++)
                 if (j!=kln) A(i,j)-=A(i,kln)*A(klm,j)/A(klm,kln);
  for(j=0;j<A.size_j();j++) if (j!=kln) A(klm,j)/=A(klm,kln);
  for(i=1;i<A.size_i();i++) A(i,kln)=0; A(klm,kln)=1;
}
       //    . 
Vector simplexMethod( SimplexIteration s)
{ ofstream out; SimplexIteration::bad_simplex e;
  if (!s.NoTable)
  { out.open(s.out_f);
    if (!out) { e.add_err(7); throw e; }
  }
  while( (s.Stop>0 && s.iter<s.Stop) || (s.Stop<=0 && size_t(s.iter)<s.A.size_i()*4) )
           { s.calculate_d();
             if (!s.be_not_optimal()) break;
             if (!s.be_solved()) { e.add_err(12); break; }
             if (!s.NoTable)
                {
                  if (s.OnlyDataTable && !s.iter) s.print(out);
                  if (!s.OnlyDataTable && !s.OnlyOptimalTable
                      && s.iter>=s.Start_Iter &&
                      (s.iter<=s.End_Iter || !s.End_Iter) ) s.print(out);
                }
             s++;
           }

  if (s.d(0,0)) e.add_err(12);

  if (!s.NoTable)
  {
    if (s.OnlyOptimalTable) s.print(out);
    if (!s.OnlyOptimalTable && !s.OnlyDataTable && s.iter>=s.Start_Iter
        && (s.iter<=s.End_Iter || !s.End_Iter) ) s.print(out);
  }

  Vector x(s.A.size_j()); x(0)=s.d(0,0)*s.M+s.d(1,0);
  for (size_t i=1;i<s.A.size_i();i++) x(s.B[i])=s.A(i,0);
  if (!s.NoTable) out.close();
  if (e) throw e; return x;
}

//----      ------------------------------

char* SimplexIteration::num(char *st,int sm,size_t n) const
{ ostrstream str(st+sm,3);
  str<<setw(3)<<setiosflags(ios::left)<<streamsize(n); return st;
}

void SimplexIteration::line(ostream &out, Border st, long i, long pe) const
{ Border z=GL; if (st==VL || st==NOP) z=NOP;
  for(size_t j;i<=pe;i++)
     { out<<char(st);
       for(j=0;j<w[i]+2;j++) out<<char(z);
     }
}

void SimplexIteration::centr(ostream &out,char *c,long w) const
{ size_t d=w+2-strlen(c), d1=d/2, d2=d%2;
  out<<setw(streamsize(d1+d2))<<' '<<c<<setw(streamsize(d1))<<' '; }

void SimplexIteration::type_d(ostream &out, double d, long i) const
{ out<<char(VL);
  if (d==M) out<<setw(streamsize(w[i]))<<' '<<"M ";
  else if (d==-M) out<<setw(streamsize(w[i]-1))<<' '<<"-M ";
       else out<<setw(streamsize(w[i]+1))<<setprecision(Precision)
               <<setiosflags(ios::fixed)<<d<<' ';
}

void SimplexIteration::line(ostream &out) const
{ line(out,CL,-2,-2); line(out,CM,-1,long(A.size_j()-1)); end_line(out,CR); }

//--------------------------------------------------------------------------

void SimplexIteration::print(ostream& out) const
{  if (!iter && name) out<<": "<<name<<endl;
   char *niter=": NNN\0()",*nx="xNNN"; size_t i,j;
   niter[13]='\0';

   //  w[-1]
   if (var) for(i=1,w[-1]=3; i<A.size_i(); i++)
               if ((j=(int)strlen(var[B[i]]))>w[-1]) w[-1]=j;

   //  
   line(out,NOP,-2,0); line(out,UL,1,1); line(out,UM,2,long(A.size_j()-1));
   end_line(out,UR);
   if (!iter) centr(out," ",long(w[-2]+w[-1]+w[0]+7));
      else { if (!(kln && klm)) niter[13]=' ';
             centr(out,num(niter,10,iter),long(w[-2]+w[-1]+w[0]+7));
           }
   for(j=1; j<A.size_j(); j++) type_d(out,A(0,j),long(j));
   end_line(out,VL);
   line(out,UL,-2,-2); line(out,UM,-1,0); line(out,CM,1,long(A.size_j()-1));
   end_line(out,CR);
   type_c(out,"C ",-2); type_c(out,"",-1); type_c(out,"X ",0);
   for(j=1; j<A.size_j(); j++)
      if (var) type_c(out,var[j],long(j)); else type_c(out,num(nx,1,j),long(j));
   end_line(out,VL); line(out);

   //  
   for(i=1;i<A.size_i();i++)
      { type_d(out,A(0,B[i]),-2);
        if (var) type_c(out,var[B[i]],-1); else type_c(out,num(nx,1,B[i]),-1);
        for(j=0; j<A.size_j(); j++) type_d(out,A(i,j),long(j));
        end_line(out,VL); if (i!=A.size_i()-1) line(out); }

   //   
   line(out,CL,-2,-2); line(out,DM,-1,-1); line(out,CM,0,long(A.size_j()-1));
   end_line(out,CR);
     out<<char(VL); centr(out,"",long(w[-2]+w[-1]+3));
     for(j=0; j<A.size_j(); j++)
        if (d(0,j)) type_d(out,d(0,j),long(j)); else line(out,VL,long(j),long(j));
     end_line(out,VL);
        out<<char(VL); centr(out,"     ",long(w[-2]+w[-1]+3));
        for(j=0; j<A.size_j(); j++) type_d(out,d(1,j),long(j));
        end_line(out,VL);
   line(out,DL,-2,-2); line(out,GL,-1,-1); line(out,DM,0,long(A.size_j()-1));
   end_line(out,DR);

   if (kln && klm) out<<" : "<<int(kln)
                      <<"   : "<<int(klm);
      else out<<"   !";
   out<<endl<<endl;
}
