#pragma once

#include "d_matrix.h" 
#include <limits.h> //  LONG_MAX

using namespace std;

//------------------------------------------------------------+
//       -  |
//------------------------------------------------------------+

const int MAX_COL=500,MAX_LINES=300, //    
          MAX_ERR=10;                // . -  

class SimplexIteration {
private:
   enum { Vacant,MAX,MIN } crit; //    
   Matrix A,  //  
          d;  //    ( )
   size_t *B,    //    
          *w,    //   
          kln,   //  
          klm,   //  
          base;  //    () 
   int    iter,  //   
          Start_Iter,    //  
          End_Iter,      // 
          Stop;          //      
   bool   OnlyDataTable,        //      
          OnlyOptimalTable,     //      .
          NoTable;              //     
   char  **var;  //    
   char  *name;  //   
   char  *out_f; //     

   int  Width,       //   
        Precision;   //   
   static double mash_eps;  //       0

//  
   enum Border { VL='|', GL='-', NOP=' ',
                 UL='+', UM='+', UR='+',
                 CL='+', CM='+', CR='+',
                 DL='+', DM='+', DR='+' };

//  
   size_t be_solved();  //   
                        //   . 
   size_t be_not_optimal();  //     
                             //   . 
   void calculate_d();    //   
   void operator++(int);  //   

//     
   char* num(char*,int,size_t) const;
   void line(ostream&, Border, long, long) const;
   void line(ostream&) const;
   void centr(ostream&,char*, long) const;
   void type_c(ostream &out, char *c, long i) const { out<<char(VL); centr(out,c,long(w[i])); }
   void type_d(ostream&, double, long) const;
   void end_line(ostream &out,Border c) const { out<<char(c)<<endl; }
   void ReWidth();
    
public:
   static const long M = LONG_MAX-10;
   SimplexIteration();            //  (  )
   SimplexIteration(const char*); //  (  )
   ~SimplexIteration();           // 
   SimplexIteration(const SimplexIteration&);
   SimplexIteration& operator=(const SimplexIteration&);

   static void epsilon(double e) { mash_eps=e; }  //  
   static double epsilon() { return mash_eps; }   //   

// exception class	
   class bad_simplex
   {   static const char *err_mas[];
	   int status[MAX_ERR]; //   
       int st_c;            // .    
	 public:
		 bad_simplex() : st_c(0) {}
		 void add_err(int n) { if (st_c<MAX_ERR) status[st_c++]=n; }

		 //   
		 operator int() const { return st_c; }
         // .    .   
		 const char* text_err(int n) const { return (n<st_c)?err_mas[status[n]]:0; }
   };

   //   
   int GetWidth() const { return Width; }
   int GetPrecision() const { return Precision; }
   void NewWidth(int w) { Width=w; ReWidth(); }
   void NewPrecision(int p) { Precision=p; }

   int Get_OnlyOptimalTable() const { return OnlyOptimalTable; }
   int Get_OnlyDataTable() const { return OnlyDataTable; }
   int Get_NoTable() const { return NoTable; }
   int Get_Start_Iter() const { return Start_Iter; }
   int Get_End_Iter() const { return End_Iter; }

   void Put_OnlyOptimalTable(bool a) { OnlyOptimalTable=a; }
   void Put_OnlyDataTable(bool a) { OnlyDataTable=a; }
   void Put_NoTable(bool a) { NoTable=a; }
   void Put_Start_Iter(int a) { Start_Iter=a; }
   void Put_End_Iter(int a) { End_Iter=a; }

   void print(ostream&) const;  //    
   void field(int w,int p=2) { Width=w; Precision=p; }

   friend Vector simplexMethod(SimplexIteration);
                          //    

}; /*  SimplexIteration */

//-------------------------------------------------------------------------

